<?php

/**
 * EstoValidationModuleFrontController Controller
 * Responsible for handling orders
 */
class EstoValidationModuleFrontController extends ModuleFrontController
{
    const STATUS_APPROVED = 'APPROVED';

    public function initContent()
    {
        # this block prevents multiple simultaneous executions by several return requests received from esto service
        $filename = 'locked';
        $file = fopen($filename, "w");
        flock($file, LOCK_EX);

        $module = $this->module;

        if(!$module->active) {
            die($module->l('This payment method is not available.'));
        }

        $data = Tools::getAllValues();
        $estoPayment = $data['esto_payment'];
        $apiCountry = $data['api_country'];

        $paymentMessage = htmlspecialchars_decode($data['json']);
        $mac = $data['mac'];

        if (!MACHelpers::verifyMessage($paymentMessage, $mac, $apiCountry, true)) {
            die($module->l('Illegal access'));
        }

        $data = json_decode($paymentMessage, true);
        $data['esto_payment'] = $estoPayment;

        if (Configuration::get(ConfigKeysHelper::ESTO_CREATE_ORDER) === CreateOrderType::AFTER) {
            if ($data['status'] === self::STATUS_APPROVED) {
                $order = $this->createOrder($data);

                $paymentState = Configuration::get(ConfigKeysHelper::PS_OS_PAYMENT);

                if ((int)$order->current_state !== (int) $paymentState) {
                    $order->setCurrentState($paymentState);
                }

                Tools::redirect($this->getOrderConfirmationLink($order));
            } else {
                Tools::redirect($this->getCartUrl());
            }
        }

        if (Configuration::get(ConfigKeysHelper::ESTO_CREATE_ORDER) === CreateOrderType::BEFORE) {
            $order = new Order((int) $data['reference']);

            if ($data['status'] === self::STATUS_APPROVED) {
                $paymentState = Configuration::get('PS_OS_PAYMENT');

                if ((int) $order->current_state !== (int) $paymentState) {
                    $order->setCurrentState($paymentState);
                }

                Tools::redirect($this->getOrderConfirmationLink($order));
            } else {
                $paymentState = Configuration::get(ConfigKeysHelper::PS_OS_CANCELED);

                if ((int) $order->current_state !== (int) $paymentState) {
                    $order->setCurrentState($paymentState);
                }

                $this->createNewCart($order);

                Tools::redirect($this->getCartUrl());
            }
        }

        // Should not get here
        Tools::redirect('index.php');
    }

    private function createOrder($data)
    {
        $module = $this->module;

        $order = null;

        $estoPayment = Configuration::get($data['esto_payment'] . '_OS_PROCESSING');
        $reference = (int) $data['reference'];

        $cart = new Cart($reference);
        $customer = new Customer($cart->id_customer);

        $message = Configuration::get(ConfigKeysHelper::ESTO_CONNECTION_MODE) === ConnectionModeType::TEST ? 'TEST MODE' : null;

        $cartOrderId = Order::getOrderByCartId($cart->id);

        if (Validate::isLoadedObject($cart) &&
            $cart->OrderExists() == true &&
            $cartOrderId &&
            ($customer->secure_key !== false && $customer->secure_key == $cart->secure_key))
        {
            $order = new Order($cartOrderId);
            return $order;
        }


        try {
            $module->validateOrder(
                $cart->id,
                $estoPayment,
                0,
                str_replace('_', ' ', $data['esto_payment']),
                $message,
                [],
                null,
                false,
                $customer->secure_key
            );
            $order = new Order($module->currentOrder);
        } catch (Exception $e) {
            //Prestashop log
        }

        return $order;
    }

    private function createNewCart($order)
    {
        $orderCart = new Cart($order->id_cart);

        $dupe = $orderCart->duplicate();

        $cart = $dupe['cart'];

        $this->context->cart = $cart;
        $this->context->cookie->__set('id_cart', (int) $cart->id);
    }

    private function getOrderConfirmationLink($order)
    {
        $cart = new Cart($order->id_cart);
        $customer = new Customer($order->id_customer);

        return $this->context->link->getPageLink(
            'order-confirmation',
            null,
            (int) $this->context->language->id,
            [
                'id_cart' => (int) $cart->id,
                'id_module' => (int) $this->module->id,
                'id_order' => (int) $order->id,
                'key' => $customer->secure_key,
            ]
        );
    }

    private function getCartUrl()
    {
        $orderType = (int) Configuration::get(ConfigKeysHelper::PS_ORDER_PROCESS_TYPE) === 1 ? 'order-opc' : 'order';

        return $this->context->link->getPageLink(
            $orderType,
            null,
            $this->context->language->id
        );
    }
}
