<?php
/**
 * 2007-2017 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2017 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * Esto Class
 * Main entry file for ESTO payment module
 */
class Esto extends PaymentModule
{
    const MIN_AMOUNT = 36;
    const ESTO_X_MIN_AMOUNT = 60;
    const MAX_AMOUNT = 10000;

    const CACHE_TIME_MINUTES = 10;

    public function __construct()
    {
        $this->name = 'esto';
        $this->tab = 'payments_gateways';
        $this->version = '1.6.11';
        $this->author = 'Mikk Mihkel Nurges, ESTO AS | Veebipoed.ee';
        $this->is_eu_compatible = 1;
        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => '1.6.99.99');
        $this->controllers = array('payment', 'validation');
        $this->bootstrap = true;
        $this->currencies = false;

        parent::__construct();

        $this->displayName = $this->l('ESTO');
        $this->description = $this->l('Payment module for paying with ESTO lease');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');

        /* Init all dependencies */
        $this->init();
    }

    /*
     * Method, that bootstraps all classes and types
     */
    private function init()
    {
        /* Types */
        include_once _PS_MODULE_DIR_ . 'esto/classes/types/AbstractType.php';
        include_once _PS_MODULE_DIR_ . 'esto/classes/types/ConnectionModeType.php';
        include_once _PS_MODULE_DIR_ . 'esto/classes/types/CreateOrderType.php';
        include_once _PS_MODULE_DIR_ . 'esto/classes/types/CountryCodeType.php';
        include_once _PS_MODULE_DIR_ . 'esto/classes/types/PaymentScheduleTypeType.php';
        include_once _PS_MODULE_DIR_ . 'esto/classes/types/PaymentTypeType.php';
        include_once _PS_MODULE_DIR_ . 'esto/classes/types/RegionType.php';
        include_once _PS_MODULE_DIR_ . 'esto/classes/types/AdminFormTabType.php';

        /* ESTO plugin install/uninstall */
        include_once _PS_MODULE_DIR_ . 'esto/classes/EstoInstall.php';

        /* Countries helper */
        include_once _PS_MODULE_DIR_ . 'esto/classes/CountryHelper.php';

        /* MAC helper */
        include_once _PS_MODULE_DIR_.'esto/classes/MACHelpers.php';

        /* ESTO API communitation */
        include_once _PS_MODULE_DIR_ . 'esto/classes/EstoAPI.php';

        /* Config Keys helper */
        include_once _PS_MODULE_DIR_ . 'esto/classes/ConfigKeysHelper.php';

        /* Payment method helper */
        include_once _PS_MODULE_DIR_ . 'esto/classes/PaymentMethodHelper.php';
    }

    /*
     * Method, that is triggered for installation
     */
    public function install()
    {
        if (
            !parent::install() ||
            !$this->registrationHook()
        ) {
            return false;
        }

        return (new EstoInstall())->install();

    }

    /*
     * Method, that is triggered for uninstall
     */
    public function uninstall()
    {

        if (!parent::uninstall()) {
            return false;
        }

        (new EstoInstall())->uninstall();

        return true;
    }

    /*
     * Method, for hooks registration
     */
    private function registrationHook()
    {
        if (!$this->registerHook('payment')
            || !$this->registerHook('displayProductPriceBlock')
            || !$this->registerHook('header')
        ) {
            return false;
        }

        return true;
    }

    /*
     * Method, for handling info update/show in admin panel
     */
    public function getContent()
    {
        $output = null;

        if (Tools::isSubmit('submit'.$this->name)) {
            $activeLangs = Language::getLanguages(true);

            $esto_shop_id_ee = strval(Tools::getValue('ESTO_SHOP_ID_EE'));
            $esto_secret_key_ee = strval(Tools::getValue('ESTO_SECRET_KEY_EE'));
            $esto_shop_id_lv = strval(Tools::getValue('ESTO_SHOP_ID_LV'));
            $esto_secret_key_lv = strval(Tools::getValue('ESTO_SECRET_KEY_LV'));
            $esto_shop_id_lt = strval(Tools::getValue('ESTO_SHOP_ID_LT'));
            $esto_secret_key_lt = strval(Tools::getValue('ESTO_SECRET_KEY_LT'));
            $esto_conn_mode = strval(Tools::getValue('ESTO_CONNECTION_MODE'));
            $esto_create_order = strval(Tools::getValue('ESTO_CREATE_ORDER'));
            $esto_country = strval(Tools::getValue('ESTO_COUNTRY'));
            $esto_show_checkout_logo = boolval(Tools::getValue('ESTO_SHOW_CHECKOUT_LOGO'));
            $esto_min_amount = floatval(Tools::getValue('ESTO_MIN_ORDER_AMOUNT'));
            $esto_max_amount = floatval(Tools::getValue('ESTO_MAX_ORDER_AMOUNT'));
            $esto_monthly_enabled = intval(Tools::getValue('ESTO_MONTHLY_ENABLED_enable_estomonthly'));
            $esto_monthly_url = strval(Tools::getValue('ESTO_MONTHLY_URL'));
            $esto_monthly_desc = strval(Tools::getValue('ESTO_MONTHLY_DESCRIPTION'));
            $esto_checkout_logo_delete = boolval(Tools::getValue('ESTO_CHECKOUT_LOGO_DELETE_delete'));
            $esto_x_enabled = intval(Tools::getValue('ESTO_X_IS_ACTIVE'));
            $esto_x_checkout_calc = intval(Tools::getValue('ESTO_X_CHECKOUT_CALC'));
            $esto_x_min_amount = max(floatval(Tools::getValue('ESTO_X_MIN_ORDER_AMOUNT')), self::ESTO_X_MIN_AMOUNT);
            $esto_x_max_amount = floatval(Tools::getValue('ESTO_X_MAX_ORDER_AMOUNT'));
            $esto_pay_later_enabled = intval(Tools::getValue('ESTO_PAY_LATER_IS_ACTIVE'));
            $esto_pay_later_min_amount = floatval(Tools::getValue('ESTO_PAY_LATER_MIN_ORDER_AMOUNT'));
            $esto_pay_later_max_amount = floatval(Tools::getValue('ESTO_PAY_LATER_MAX_ORDER_AMOUNT'));
            $esto_pay_enabled = intval(Tools::getValue('ESTO_PAY_IS_ACTIVE'));
            $esto_pay_card_payment_enabled = intval(Tools::getValue('ESTO_PAY_CARD_PAYMENT_IS_ACTIVE'));
            $esto_use_specific_endpoint_ee = boolval(Tools::getValue('ESTO_USE_SPECIFIC_ENDPOINT_EE'));
            $esto_use_specific_endpoint_lv = boolval(Tools::getValue('ESTO_USE_SPECIFIC_ENDPOINT_LV'));
            $esto_use_specific_endpoint_lt = boolval(Tools::getValue('ESTO_USE_SPECIFIC_ENDPOINT_LT'));


            if(!$this->validateCredentials($esto_country)){
                $output .= $this->displayError(
                    $this->l('Please fill Shop Id and Secret key for selected primary endpoint country')
                );
            } elseif (!$this->validateSpecificCountryCredentials(
                $esto_use_specific_endpoint_ee, $esto_use_specific_endpoint_lv, $esto_use_specific_endpoint_lt
            )) {
                $output .= $this->displayError(
                    $this->l('Please fill Shop Id and Secret key for all selected specific endpoint countries')
                );
            } elseif (is_null($esto_conn_mode)
                || !$esto_min_amount
                || !$esto_max_amount
            ) {
                $output .= $this->displayError($this->l('Invalid Configuration value'));
            } else {
                Configuration::updateValue('ESTO_SHOP_ID_EE', $esto_shop_id_ee);
                Configuration::updateValue('ESTO_SECRET_KEY_EE', $esto_secret_key_ee);
                Configuration::updateValue('ESTO_SHOP_ID_LV', $esto_shop_id_lv);
                Configuration::updateValue('ESTO_SECRET_KEY_LV', $esto_secret_key_lv);
                Configuration::updateValue('ESTO_SHOP_ID_LT', $esto_shop_id_lt);
                Configuration::updateValue('ESTO_SECRET_KEY_LT', $esto_secret_key_lt);
                Configuration::updateValue('ESTO_CONNECTION_MODE', $esto_conn_mode);
                Configuration::updateValue('ESTO_CREATE_ORDER', $esto_create_order);
                Configuration::updateValue('ESTO_COUNTRY', $esto_country);
                Configuration::updateValue('ESTO_SHOW_CHECKOUT_LOGO', $esto_show_checkout_logo);
                Configuration::updateValue('ESTO_MIN_ORDER_AMOUNT', $esto_min_amount);
                Configuration::updateValue('ESTO_MAX_ORDER_AMOUNT', $esto_max_amount);
                Configuration::updateValue('ESTO_MONTHLY_ENABLED_enable_estomonthly', $esto_monthly_enabled);
                Configuration::updateValue('ESTO_CHECKOUT_LOGO_DELETE_delete', $esto_checkout_logo_delete);
                Configuration::updateValue('ESTO_X_IS_ACTIVE', $esto_x_enabled);
                Configuration::updateValue('ESTO_X_CHECKOUT_CALC', $esto_x_checkout_calc);
                Configuration::updateValue('ESTO_X_MIN_ORDER_AMOUNT', $esto_x_min_amount);
                Configuration::updateValue('ESTO_X_MAX_ORDER_AMOUNT', $esto_x_max_amount);
                Configuration::updateValue('ESTO_PAY_LATER_IS_ACTIVE', $esto_pay_later_enabled);
                Configuration::updateValue('ESTO_PAY_LATER_MIN_ORDER_AMOUNT', $esto_pay_later_min_amount);
                Configuration::updateValue('ESTO_PAY_LATER_MAX_ORDER_AMOUNT', $esto_pay_later_max_amount);
                Configuration::updateValue('ESTO_PAY_IS_ACTIVE', $esto_pay_enabled);
                Configuration::updateValue('ESTO_PAY_CARD_PAYMENT_IS_ACTIVE', $esto_pay_card_payment_enabled);
                Configuration::updateValue('ESTO_USE_SPECIFIC_ENDPOINT_EE', $esto_use_specific_endpoint_ee);
                Configuration::updateValue('ESTO_USE_SPECIFIC_ENDPOINT_LV', $esto_use_specific_endpoint_lv);
                Configuration::updateValue('ESTO_USE_SPECIFIC_ENDPOINT_LT', $esto_use_specific_endpoint_lt);


                $values = [];
                $keys = [
                    ConfigKeysHelper::ESTO_DISPLAY_NAME                         => ['lang' => true, 'type' => 'str', 'html' => false],
                    ConfigKeysHelper::ESTO_DISPLAY_DESCRIPTION                  => ['lang' => true, 'type' => 'str', 'html' => true],
                    ConfigKeysHelper::ESTO_X_DISPLAY_NAME                       => ['lang' => true, 'type' => 'str', 'html' => false],
                    ConfigKeysHelper::ESTO_X_DISPLAY_DESCRIPTION                => ['lang' => true, 'type' => 'str', 'html' => true],
                    ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_NAME               => ['lang' => true, 'type' => 'str', 'html' => false],
                    ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_DESCRIPTION        => ['lang' => true, 'type' => 'str', 'html' => true],
                    ConfigKeysHelper::ESTO_PAY_DISPLAY_NAME                     => ['lang' => true, 'type' => 'str', 'html' => false],
                    ConfigKeysHelper::ESTO_PAY_DISPLAY_DESCRIPTION              => ['lang' => true, 'type' => 'str', 'html' => true],
                    ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_NAME        => ['lang' => true, 'type' => 'str', 'html' => false],
                    ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_DESCRIPTION => ['lang' => true, 'type' => 'str', 'html' => true],
                    ConfigKeysHelper::ESTO_MONTHLY_URL                          => ['lang' => true, 'type' => 'str', 'html' => false],
                    ConfigKeysHelper::ESTO_MONTHLY_DESCRIPTION                  => ['lang' => true, 'type' => 'str', 'html' => true],
                ];

                // Get data sent from $_POST
                foreach ($keys as $key => $info) {
                    $functionName = $info['type'] . 'val'; // Either strval, floatval or intval

                    if(!isset($info['lang']) || $info['lang'] === false) {
                        continue;
                    }

                    if(!isset($values[$key])) {
                        $values[$key] = [];
                    }

                    foreach($activeLangs as $lang) {
                        $values[$key][$lang['id_lang']] = $functionName(Tools::getValue($key . '_' . $lang['id_lang']));
                    }
                }

                // Update configuration
                foreach ($keys as $key => $info) {
                    Configuration::updateValue($key, $values[$key], $info['html']);
                }

                // Save & delete logos
                foreach($activeLangs as $lang) {
                    $output .= $this->updateLogo(ConfigKeysHelper::ESTO_CHECKOUT_LOGO, $lang['id_lang']);
                    $output .= $this->updateLogo(ConfigKeysHelper::ESTO_X_CHECKOUT_LOGO, $lang['id_lang']);
                    $output .= $this->updateLogo(ConfigKeysHelper::ESTO_PAY_LATER_CHECKOUT_LOGO, $lang['id_lang']);
                    $output .= $this->updateLogo(ConfigKeysHelper::ESTO_PAY_CHECKOUT_LOGO, $lang['id_lang']);
                    $output .= $this->updateLogo(ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_CHECKOUT_LOGO, $lang['id_lang']);
                    $output .= $this->updateLogo(ConfigKeysHelper::ESTO_MONTHLY_LOGO, $lang['id_lang']);
                }

                $this->deleteLogo(ConfigKeysHelper::ESTO_CHECKOUT_LOGO);
                $this->deleteLogo(ConfigKeysHelper::ESTO_X_CHECKOUT_LOGO);
                $this->deleteLogo(ConfigKeysHelper::ESTO_PAY_LATER_CHECKOUT_LOGO);
                $this->deleteLogo(ConfigKeysHelper::ESTO_MONTHLY_LOGO);
                $this->deleteLogo(ConfigKeysHelper::ESTO_PAY_CHECKOUT_LOGO);
                $this->deleteLogo(ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_CHECKOUT_LOGO);

                // Save enabled languages
                $this->updateEnabledLanguages();

                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }

            // clear cache for banks in supported country
            $sql = 'DELETE FROM `'._DB_PREFIX_.'esto_banks`';
            Db::getInstance()->Execute($sql);
            // clear cache for banks in supported country
        }

        return $output . $this->displayForm();
    }

    private function displayForm()
    {
        // Get default language
        $defaultLang = (int) Configuration::get('PS_LANG_DEFAULT');
        $activeLangs = Language::getLanguages(true);

        // Init Fields form array
        $fieldsForm[0]['form'] = $this->getConfigForm();

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        // Language
        $helper->languages = $this->context->controller->getlanguages();
        $helper->default_form_language = $defaultLang;
        $helper->allow_employee_form_lang = $defaultLang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action = 'submit' . $this->name;
        $helper->toolbar_btn = array(
            'save' => array(
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                    '&token='.Tools::getAdminTokenLite('AdminModules'),
            ),
            'back' => array(
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list'),
            ),
        );

        // Load current values
        $helper->fields_value[ConfigKeysHelper::ESTO_SHOP_ID_EE] = Configuration::get(ConfigKeysHelper::ESTO_SHOP_ID_EE);
        $helper->fields_value[ConfigKeysHelper::ESTO_SECRET_KEY_EE] = Configuration::get(ConfigKeysHelper::ESTO_SECRET_KEY_EE);
        $helper->fields_value[ConfigKeysHelper::ESTO_SHOP_ID_LV] = Configuration::get(ConfigKeysHelper::ESTO_SHOP_ID_LV);
        $helper->fields_value[ConfigKeysHelper::ESTO_SECRET_KEY_LV] = Configuration::get(ConfigKeysHelper::ESTO_SECRET_KEY_LV);
        $helper->fields_value[ConfigKeysHelper::ESTO_SHOP_ID_LT] = Configuration::get(ConfigKeysHelper::ESTO_SHOP_ID_LT);
        $helper->fields_value[ConfigKeysHelper::ESTO_SECRET_KEY_LT] = Configuration::get(ConfigKeysHelper::ESTO_SECRET_KEY_LT);
        $helper->fields_value[ConfigKeysHelper::ESTO_CONNECTION_MODE] = Configuration::get(ConfigKeysHelper::ESTO_CONNECTION_MODE);
        $helper->fields_value[ConfigKeysHelper::ESTO_CREATE_ORDER] = Configuration::get(ConfigKeysHelper::ESTO_CREATE_ORDER);
        $helper->fields_value[ConfigKeysHelper::ESTO_COUNTRY] = Configuration::get(ConfigKeysHelper::ESTO_COUNTRY);
        $helper->fields_value[ConfigKeysHelper::ESTO_SHOW_CHECKOUT_LOGO] = Configuration::get(ConfigKeysHelper::ESTO_SHOW_CHECKOUT_LOGO);
        $helper->fields_value[ConfigKeysHelper::ESTO_MIN_ORDER_AMOUNT] = Configuration::get(ConfigKeysHelper::ESTO_MIN_ORDER_AMOUNT);
        $helper->fields_value[ConfigKeysHelper::ESTO_MAX_ORDER_AMOUNT] = Configuration::get(ConfigKeysHelper::ESTO_MAX_ORDER_AMOUNT);
        $helper->fields_value[ConfigKeysHelper::ESTO_MONTHLY_ENABLED_enable_estomonthly] = Configuration::get(ConfigKeysHelper::ESTO_MONTHLY_ENABLED_enable_estomonthly);
        
        $helper->fields_value[ConfigKeysHelper::ESTO_CHECKOUT_LOGO_DELETE_delete] = Configuration::get(ConfigKeysHelper::ESTO_CHECKOUT_LOGO_DELETE_delete);
        $helper->fields_value[ConfigKeysHelper::ESTO_X_IS_ACTIVE] = Configuration::get(ConfigKeysHelper::ESTO_X_IS_ACTIVE);
        $helper->fields_value[ConfigKeysHelper::ESTO_X_MIN_ORDER_AMOUNT] = Configuration::get(ConfigKeysHelper::ESTO_X_MIN_ORDER_AMOUNT);
        $helper->fields_value[ConfigKeysHelper::ESTO_X_MAX_ORDER_AMOUNT] = Configuration::get(ConfigKeysHelper::ESTO_X_MAX_ORDER_AMOUNT);
        $helper->fields_value[ConfigKeysHelper::ESTO_X_CHECKOUT_CALC] = Configuration::get(ConfigKeysHelper::ESTO_X_CHECKOUT_CALC);
        
        $helper->fields_value[ConfigKeysHelper::ESTO_PAY_LATER_IS_ACTIVE] = Configuration::get(ConfigKeysHelper::ESTO_PAY_LATER_IS_ACTIVE);
        $helper->fields_value[ConfigKeysHelper::ESTO_PAY_LATER_MIN_ORDER_AMOUNT] = Configuration::get(ConfigKeysHelper::ESTO_PAY_LATER_MIN_ORDER_AMOUNT);
        $helper->fields_value[ConfigKeysHelper::ESTO_PAY_LATER_MAX_ORDER_AMOUNT] = Configuration::get(ConfigKeysHelper::ESTO_PAY_LATER_MAX_ORDER_AMOUNT);
        
        $helper->fields_value[ConfigKeysHelper::ESTO_PAY_IS_ACTIVE] = Configuration::get(ConfigKeysHelper::ESTO_PAY_IS_ACTIVE);

        $helper->fields_value[ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_IS_ACTIVE] = Configuration::get(ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_IS_ACTIVE);

        $helper->fields_value[ConfigKeysHelper::ESTO_X_CHECKOUT_LOGO_DELETE] = false;
        $helper->fields_value[ConfigKeysHelper::ESTO_PAY_LATER_CHECKOUT_LOGO_DELETE] = false;
        $helper->fields_value[ConfigKeysHelper::ESTO_PAY_CHECKOUT_LOGO_DELETE] = false;
        $helper->fields_value[ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_CHECKOUT_LOGO_DELETE] = false;

        $helper->fields_value[ConfigKeysHelper::ESTO_USE_SPECIFIC_ENDPOINT_EE] = Configuration::get(ConfigKeysHelper::ESTO_USE_SPECIFIC_ENDPOINT_EE);
        $helper->fields_value[ConfigKeysHelper::ESTO_USE_SPECIFIC_ENDPOINT_LV] = Configuration::get(ConfigKeysHelper::ESTO_USE_SPECIFIC_ENDPOINT_LV);
        $helper->fields_value[ConfigKeysHelper::ESTO_USE_SPECIFIC_ENDPOINT_LT] = Configuration::get(ConfigKeysHelper::ESTO_USE_SPECIFIC_ENDPOINT_LT);


        foreach ($activeLangs as $lang) {
            $helper->fields_value[ConfigKeysHelper::ESTO_MONTHLY_ENABLED_LANGS_id . $lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_MONTHLY_ENABLED_LANGS_id . $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_DISPLAY_NAME][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_DISPLAY_NAME, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_DISPLAY_DESCRIPTION][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_DISPLAY_DESCRIPTION, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_X_DISPLAY_NAME][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_X_DISPLAY_NAME, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_X_DISPLAY_DESCRIPTION][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_X_DISPLAY_DESCRIPTION, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_NAME][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_NAME, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_DESCRIPTION][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_DESCRIPTION, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_PAY_DISPLAY_NAME][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_PAY_DISPLAY_NAME, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_PAY_DISPLAY_DESCRIPTION][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_PAY_DISPLAY_DESCRIPTION, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_NAME][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_NAME, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_DESCRIPTION][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_DESCRIPTION, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_CHECKOUT_LOGO][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_CHECKOUT_LOGO, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_X_CHECKOUT_LOGO][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_X_CHECKOUT_LOGO, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_PAY_LATER_CHECKOUT_LOGO][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_PAY_LATER_CHECKOUT_LOGO, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_PAY_CHECKOUT_LOGO][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_PAY_CHECKOUT_LOGO, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_CHECKOUT_LOGO][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_CHECKOUT_LOGO, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_MONTHLY_URL][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_MONTHLY_URL, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_MONTHLY_DESCRIPTION][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_MONTHLY_DESCRIPTION, $lang['id_lang']);
            $helper->fields_value[ConfigKeysHelper::ESTO_MONTHLY_LOGO][$lang['id_lang']] = Configuration::get(ConfigKeysHelper::ESTO_MONTHLY_LOGO, $lang['id_lang']);
            
        }

        return $helper->generateForm($fieldsForm);
    }

    private function getConfigForm()
    {
        $country_array = [
            [
                'esto_name' => RegionType::ESTONIA,
                'name' => $this->l(Tools::ucfirst(RegionType::ESTONIA))
            ],
            [
                'esto_name' => RegionType::LITHUANIA,
                'name' => $this->l(Tools::ucfirst(RegionType::LITHUANIA))
            ],
            [
                'esto_name' => RegionType::LATVIA,
                'name'=>$this->l(Tools::ucfirst(RegionType::LATVIA))
            ],
        ];

        $activeLangs = Language::getLanguages(true);
        $langSet = [];

        foreach ($activeLangs as $lang) {
            $langSet[] = [
                'id' => 'id' . $lang['id_lang'],
                'name' => $lang['name'],
                'val' => '1',
                'checked' => '',
            ];
        }

        $form = [
            'tabs' => [
                AdminFormTabType::GENERAL => $this->l('General settings'),
                AdminFormTabType::REGULAR => $this->l('ESTO'),
                AdminFormTabType::ESTO_X => $this->l('ESTO X'),
                AdminFormTabType::PAY_LATER => $this->l('ESTO Pay Later'),
                AdminFormTabType::ESTO_PAY_BANKLINK => $this->l('ESTO Pay'),
                AdminFormTabType::ESTO_PAY_CARD => $this->l('ESTO Pay (Visa/Mastercard)'),
            ],
            'tinymce' => true,
            'legend' => [
                'title' => $this->l('MODULE SETTINGS'),
                'icon'  => 'icon-cogs',
            ],
            'input' => [],
            'submit' => [
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right',
            ],
        ];

        $formGeneral = [
            [
                'tab' => AdminFormTabType::GENERAL,
                'type' => 'select',
                'label' => $this->l('Connection mode'),
                'hint' => $this->l('Use test mode for development purposes only'),
                'name' => ConfigKeysHelper::ESTO_CONNECTION_MODE,
                'required' => true,
                'options' => [
                    'query' => [
                        [
                            'id' => ConnectionModeType::TEST,
                            'name' => $this->l('Test'),
                        ],
                        [
                            'id' => ConnectionModeType::LIVE,
                            'name' => $this->l('Live'),
                        ]
                    ],
                    'id' => 'id',
                    'name' => 'name',
                ],
            ],
            [
                'tab' => AdminFormTabType::GENERAL,
                'type' => 'text',
                'label' => $this->l('Estonian endpoint Shop ID'),
                'desc' => $this->l('Shop ID is set by ESTO and can be retrieved from') . ' <a href="https://affiliate.esto.ee/developer-tools/setup" target="_blank">https://affiliate.esto.ee/developer-tools/setup</a>',
                'name' => ConfigKeysHelper::ESTO_SHOP_ID_EE,
                'size' => 10,
                'required' => false,
            ],
            [
                'tab' => AdminFormTabType::GENERAL,
                'type' => 'text',
                'label' => $this->l('Estonian endpoint Secret key'),
                'desc' => $this->l('Secret API key should be kept secret and it can be obtained from') . ' <a href="https://affiliate.esto.ee/developer-tools/setup" target="_blank">https://affiliate.esto.ee/developer-tools/setup</a>',
                'name' => ConfigKeysHelper::ESTO_SECRET_KEY_EE,
                'size' => 10,
                'required' => false,
            ],
            [
                'tab' => AdminFormTabType::GENERAL,
                'type' => 'text',
                'label' => $this->l('Latvian endpoint Shop ID'),
                'desc' => $this->l('Shop ID is set by ESTO and can be retrieved from') . ' <a href="https://affiliate.esto.lv/developer-tools/setup" target="_blank">https://affiliate.esto.lv/developer-tools/setup</a>',
                'name' => ConfigKeysHelper::ESTO_SHOP_ID_LV,
                'size' => 10,
                'required' => false,
            ],
            [
                'tab' => AdminFormTabType::GENERAL,
                'type' => 'text',
                'label' => $this->l('Latvian endpoint Secret key'),
                'desc' => $this->l('Secret API key should be kept secret and it can be obtained from') . ' <a href="https://affiliate.esto.lv/developer-tools/setup" target="_blank">https://affiliate.esto.lv/developer-tools/setup</a>',
                'name' => ConfigKeysHelper::ESTO_SECRET_KEY_LV,
                'size' => 10,
                'required' => false,
            ],
            [
                'tab' => AdminFormTabType::GENERAL,
                'type' => 'text',
                'label' => $this->l('Lithuanian endpoint Shop ID'),
                'desc' => $this->l('Shop ID is set by ESTO and can be retrieved from') . ' <a href="https://affiliate.estopay.lt/developer-tools/setup" target="_blank">https://affiliate.estopay.lt/developer-tools/setup</a>',
                'name' => ConfigKeysHelper::ESTO_SHOP_ID_LT,
                'size' => 10,
                'required' => false,
            ],
            [
                'tab' => AdminFormTabType::GENERAL,
                'type' => 'text',
                'label' => $this->l('Lithuanian endpoint Secret key'),
                'desc' => $this->l('Secret API key should be kept secret and it can be obtained from') . ' <a href="https://affiliate.estopay.lt/developer-tools/setup" target="_blank">https://affiliate.estopay.lt/developer-tools/setup</a>',
                'name' => ConfigKeysHelper::ESTO_SECRET_KEY_LT,
                'size' => 10,
                'required' => false,
            ],
            [
                'tab' => AdminFormTabType::GENERAL,
                'type'=> 'select',
                'label' => $this->l('Create order'),
                'name' => ConfigKeysHelper::ESTO_CREATE_ORDER,
                'required' => true,
                'options' => [
                    'query' => [
                        [
                            'id' => CreateOrderType::BEFORE,
                            'name' => $this->l('Before signing')
                        ],
                        [
                            'id' => CreateOrderType::AFTER,
                            'name' => $this->l('After signing')
                        ]
                    ],
                    'id' => 'id',
                    'name' => 'name'
                ]
            ],
            [
                'tab' => AdminFormTabType::GENERAL,
                'type' => 'select',
                'label' => $this->l('Primary endpoint'),
                'desc' => $this->l('Select correct endpoint for your country'),
                'name' => ConfigKeysHelper::ESTO_COUNTRY,
                'required' => true,
                'options' => array(
                    'query' => $country_array,
                    'id' => 'esto_name',
                    'name' => 'name',
                ),
            ],
            [
                'tab' => AdminFormTabType::GENERAL,
                'type' => 'checkbox',
                'label' => $this->l('Use specific endpoint?'),
                'name' => ConfigKeysHelper::ESTO_USE_SPECIFIC_ENDPOINT,
                'values' => [
                    'query' => [
                        [
                            'id' => CountryHelper::API_COUNTRIES[RegionType::ESTONIA],
                            'name' => $this->l('for Estonia'),
                            'val' => '1',
                            'checked' => '',
                        ],
                        [
                            'id' => CountryHelper::API_COUNTRIES[RegionType::LATVIA],
                            'name' => $this->l('for Latvia'),
                            'val' => '1',
                            'checked' => '',
                        ],
                        [
                            'id' => CountryHelper::API_COUNTRIES[RegionType::LITHUANIA],
                            'name' => $this->l('for Lithuania'),
                            'val' => '1',
                            'checked' => '',
                        ],
                    ],
                    'id' => 'id',
                    'name' => 'name',
                ],
            ],
            [
                'tab' => AdminFormTabType::GENERAL,
                'type' => 'switch',
                'label' => $this->l('Show ESTO logos at checkout'),
                'name' => ConfigKeysHelper::ESTO_SHOW_CHECKOUT_LOGO,
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
        ];

        $formRegular = [
            [
                'tab' => AdminFormTabType::REGULAR,
                'type' => 'checkbox',
                'label' => $this->l('Enabled in languages'),
                'name' => ConfigKeysHelper::ESTO_MONTHLY_ENABLED_LANGS,
                'values' => [
                    'query' => $langSet,
                    'id' => 'id',
                    'name' => 'name',
                ],
            ],
            [
                'tab' => AdminFormTabType::REGULAR,
                'type' => 'text',
                'label' => $this->l('Esto lease name'),
                'name' => ConfigKeysHelper::ESTO_DISPLAY_NAME,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::REGULAR,
                'type' => 'text',
                'label' => $this->l('Esto lease description'),
                'name' => ConfigKeysHelper::ESTO_DISPLAY_DESCRIPTION,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::REGULAR,
                'type' => 'text',
                'label' => $this->l('Min. order amount'),
                'hint' => $this->l('The minimum order amount required for displaying the module in checkout'),
                'name' => ConfigKeysHelper::ESTO_MIN_ORDER_AMOUNT,
                'size' => 10,
                'required' => true,
            ],
            [
                'tab' => AdminFormTabType::REGULAR,
                'type' => 'text',
                'label' => $this->l('Max. order amount'),
                'hint' => $this->l('The maximum order amount allowed for displaying the module in checkout'),
                'name' => ConfigKeysHelper::ESTO_MAX_ORDER_AMOUNT,
                'size' => 10,
                'required' => true,
            ],
            [
                'tab' => AdminFormTabType::REGULAR,
                'type' => 'file_lang',
                'label' => $this->l('Checkout logo'),
                'name' => ConfigKeysHelper::ESTO_CHECKOUT_LOGO,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::REGULAR,
                'type' => 'checkbox',
                'label' => $this->l('Delete checkout logo?'),
                'name' => ConfigKeysHelper::ESTO_CHECKOUT_LOGO_DELETE,
                'values' => [
                    'query' => [
                        [
                            'id' => 'delete',
                            'name' => '',
                            'val' => '1',
                            'checked' => '',
                        ],
                    ],
                    'id' => 'id',
                    'name' => 'name',
                ],
            ],
            [
                'tab' => AdminFormTabType::REGULAR,
                'type' => 'checkbox',
                'label' => $this->l('Enable monthly payment price on product page'),
                'name' => ConfigKeysHelper::ESTO_MONTHLY_ENABLED,
                'values' => [
                    'query' => [
                        [
                            'id' => 'enable_estomonthly',
                            'name' => '',
                            'val' => '1',
                            'checked' => '',
                        ],
                    ],
                    'id' => 'id',
                    'name' => 'name',
                ],
            ],
            [
                'tab' => AdminFormTabType::REGULAR,
                'type' => 'text',
                'label' => $this->l('Monthly payment info URL'),
                'desc' => $this->l('Must be a full URL if used (e.g. "https://www.esto.ee")'),
                'name' => ConfigKeysHelper::ESTO_MONTHLY_URL,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::REGULAR,
                'type' => 'textarea',
                'label' => $this->l('Custom text under monthly payment'),
                'name' => ConfigKeysHelper::ESTO_MONTHLY_DESCRIPTION,
                'autoload_rte' => true,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::REGULAR,
                'type' => 'file_lang',
                'label' => $this->l('Monthly payment logo'),
                'name' => ConfigKeysHelper::ESTO_MONTHLY_LOGO,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::REGULAR,
                'type' => 'checkbox',
                'label' => $this->l('Delete monthly logo?'),
                'name' => ConfigKeysHelper::ESTO_MONTHLY_LOGO_DELETE,
                'values' => [
                    'query' => [
                        [
                            'id' => 'delete',
                            'name' => '',
                            'val' => '1',
                            'checked' => '',
                        ],
                    ],
                    'id' => 'id',
                    'name' => 'name',
                ],
            ],
        ];

        $formEstoX = [
            [
                'tab' => AdminFormTabType::ESTO_X,
                'type' => 'switch',
                'label' => $this->l('Enable ESTO X payment option'),
                'name' => ConfigKeysHelper::ESTO_X_IS_ACTIVE,
                'hint' => $this->l('ESTO X is a campaign of ESTO. Contact ESTO support for additional information.'),
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
            [
                'tab' => AdminFormTabType::ESTO_X,
                'type' => 'text',
                'label' => $this->l('Esto X name'),
                'name' => ConfigKeysHelper::ESTO_X_DISPLAY_NAME,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::ESTO_X,
                'type' => 'text',
                'label' => $this->l('Esto X description'),
                'name' => ConfigKeysHelper::ESTO_X_DISPLAY_DESCRIPTION,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::ESTO_X,
                'type' => 'text',
                'label' => $this->l('Min. order amount'),
                'hint' => $this->l('The minimum order amount required for displaying the module in checkout'),
                'name' => ConfigKeysHelper::ESTO_X_MIN_ORDER_AMOUNT,
                'size' => 10,
                'required' => true,
            ],
            [
                'tab' => AdminFormTabType::ESTO_X,
                'type' => 'text',
                'label' => $this->l('Max. order amount'),
                'hint' => $this->l('The maximum order amount allowed for displaying the module in checkout'),
                'name' => ConfigKeysHelper::ESTO_X_MAX_ORDER_AMOUNT,
                'size' => 10,
                'required' => true,
            ],
            [
                'tab' => AdminFormTabType::ESTO_X,
                'type' => 'file_lang',
                'label' => $this->l('Checkout logo'),
                'name' => ConfigKeysHelper::ESTO_X_CHECKOUT_LOGO,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::ESTO_X,
                'type' => 'switch',
                'label' => $this->l('Delete checkout logo?'),
                'name' => ConfigKeysHelper::ESTO_X_CHECKOUT_LOGO_DELETE,
                'values' => [
                    [
                        'id' => 'delete_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'delete_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
            [
                'tab' => AdminFormTabType::ESTO_X,
                'type' => 'switch',
                'label' => $this->l('Enable checkout calculator'),
                'name' => ConfigKeysHelper::ESTO_X_CHECKOUT_CALC,
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
        ];

        $formEstoPayLater = [
            [
                'tab' => AdminFormTabType::PAY_LATER,
                'type' => 'switch',
                'label' => $this->l('Enable ESTO Pay Later payment option'),
                'name' => ConfigKeysHelper::ESTO_PAY_LATER_IS_ACTIVE,
                'hint' => $this->l('Pay Later is a campaign of ESTO. Contact ESTO support for additional information.'),
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
            [
                'tab' => AdminFormTabType::PAY_LATER,
                'type' => 'text',
                'label' => $this->l('Esto pay later name'),
                'name' => ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_NAME,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::PAY_LATER,
                'type' => 'text',
                'label' => $this->l('Esto pay later description'),
                'name' => ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_DESCRIPTION,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::PAY_LATER,
                'type' => 'text',
                'label' => $this->l('Min. order amount'),
                'hint' => $this->l('The minimum order amount required for displaying the module in checkout'),
                'name' => ConfigKeysHelper::ESTO_PAY_LATER_MIN_ORDER_AMOUNT,
                'size' => 10,
                'required' => true,
            ],
            [
                'tab' => AdminFormTabType::PAY_LATER,
                'type' => 'text',
                'label' => $this->l('Max. order amount'),
                'hint' => $this->l('The maximum order amount allowed for displaying the module in checkout'),
                'name' => ConfigKeysHelper::ESTO_PAY_LATER_MAX_ORDER_AMOUNT,
                'size' => 10,
                'required' => true,
            ],
            [
                'tab' => AdminFormTabType::PAY_LATER,
                'type' => 'file_lang',
                'label' => $this->l('Checkout logo'),
                'name' => ConfigKeysHelper::ESTO_PAY_LATER_CHECKOUT_LOGO,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::PAY_LATER,
                'type' => 'switch',
                'label' => $this->l('Delete checkout logo?'),
                'name' => ConfigKeysHelper::ESTO_PAY_LATER_CHECKOUT_LOGO_DELETE,
                'values' => [
                    [
                        'id' => 'delete_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'delete_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
        ];

        $formEstoPayBanklink = [
            [
                'tab' => AdminFormTabType::ESTO_PAY_BANKLINK,
                'type' => 'switch',
                'label' => $this->l('Enable ESTO Pay payment option'),
                'name' => ConfigKeysHelper::ESTO_PAY_IS_ACTIVE,
                'hint' => $this->l('ESTO Pay is a direct payment method for credit cards, banklinks, etc. Contact ESTO support for additional information.'),
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
            [
                'tab' => AdminFormTabType::ESTO_PAY_BANKLINK,
                'type' => 'text',
                'label' => $this->l('Esto pay name'),
                'name' => ConfigKeysHelper::ESTO_PAY_DISPLAY_NAME,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::ESTO_PAY_BANKLINK,
                'type' => 'text',
                'label' => $this->l('Esto pay description'),
                'name' => ConfigKeysHelper::ESTO_PAY_DISPLAY_DESCRIPTION,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::ESTO_PAY_BANKLINK,
                'type' => 'file_lang',
                'label' => $this->l('Checkout logo'),
                'name' => ConfigKeysHelper::ESTO_PAY_CHECKOUT_LOGO,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::ESTO_PAY_BANKLINK,
                'type' => 'switch',
                'label' => $this->l('Delete checkout logo?'),
                'name' => ConfigKeysHelper::ESTO_PAY_CHECKOUT_LOGO_DELETE,
                'values' => [
                    [
                        'id' => 'delete_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'delete_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
        ];

        $formEstoPayCard = [
            [
                'tab' => AdminFormTabType::ESTO_PAY_CARD,
                'type' => 'switch',
                'label' => $this->l('Enable ESTO Pay (Card payment) payment option'),
                'name' => ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_IS_ACTIVE,
                'hint' => $this->l('ESTO Pay is a direct payment method for credit cards, banklinks, etc. Contact ESTO support for additional information.'),
                'values' => [
                    [
                        'id' => 'active_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'active_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
            [
                'tab' => AdminFormTabType::ESTO_PAY_CARD,
                'type' => 'text',
                'label' => $this->l('Esto pay (Visa/Mastercard) name'),
                'name' => ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_NAME,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::ESTO_PAY_CARD,
                'type' => 'text',
                'label' => $this->l('Esto pay (Visa/Mastercard) description'),
                'name' => ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_DESCRIPTION,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::ESTO_PAY_CARD,
                'type' => 'file_lang',
                'label' => $this->l('Checkout logo'),
                'name' => ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_CHECKOUT_LOGO,
                'lang' => true,
            ],
            [
                'tab' => AdminFormTabType::ESTO_PAY_CARD,
                'type' => 'switch',
                'label' => $this->l('Delete checkout logo?'),
                'name' => ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_CHECKOUT_LOGO_DELETE,
                'values' => [
                    [
                        'id' => 'delete_on',
                        'value' => true,
                        'checked' => $this->l('Enabled'),
                    ],
                    [
                        'id' => 'delete_off',
                        'value' => false,
                        'checked' => $this->l('Disabled'),
                    ],
                ],
            ],
        ];
        
        $form['input'] = array_merge($form['input'], $formGeneral, $formRegular, $formEstoX, $formEstoPayLater, $formEstoPayBanklink, $formEstoPayCard);

        return $form;
    }

    public function hookHeader()
    {
        $this->context->controller->addCss($this->_path . 'views/css/checkout.css');
    }

    public function hookPayment($params)
    {
        $htmlPayments = '';

        $configurations = Configuration::getMultiple([
            ConfigKeysHelper::ESTO_PAY_DISPLAY_NAME,
            ConfigKeysHelper::ESTO_PAY_DISPLAY_DESCRIPTION,
            ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_NAME,
            ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_DESCRIPTION,
            ConfigKeysHelper::ESTO_DISPLAY_NAME,
            ConfigKeysHelper::ESTO_DISPLAY_DESCRIPTION,
            ConfigKeysHelper::ESTO_X_DISPLAY_NAME,
            ConfigKeysHelper::ESTO_X_DISPLAY_DESCRIPTION,
            ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_NAME,
            ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_DESCRIPTION,
        ], $this->context->language->id);


        $sql = 'select * FROM `'._DB_PREFIX_.'esto_banks`';
        $result = Db::getInstance()->getRow($sql);


        $dateUpdate = isset($result['date_update']) ? $result['date_update'] : 0;
        $idItem = isset($result['id']) ? $result['id'] : null;
        if ((strtotime(date("Y-m-d H:i:s")) - strtotime($dateUpdate)) > self::CACHE_TIME_MINUTES * 60) {

            $apiDetails = self::getDefaultCountryApiData();
            $api = new EstoAPI($apiDetails['url'], $apiDetails['user'], $apiDetails['pass'], $apiDetails['country_iso']);
            $dataBanks = $api->getBanklinks();

            if($idItem){
                $sql = 'DELETE FROM `'._DB_PREFIX_.'esto_banks` WHERE id =  '.(int)$idItem.'';
                Db::getInstance()->Execute($sql);
            }

            $sql = 'INSERT INTO `'._DB_PREFIX_.'esto_banks` SET `data` = \''.pSQL(serialize($dataBanks)).'\', date_update = \''.pSQL(date("Y-m-d H:i:s")).'\'';
		    Db::getInstance()->Execute($sql);

        } else {
            $dataBanks = isset($result['data']) ? unserialize($result['data']) : array();
        }

        $defaultCountry = CountryHelper::API_COUNTRIES[Configuration::get(ConfigKeysHelper::ESTO_COUNTRY)];
        if (!empty($params['cart']->id_address_invoice)) {
            $billingAddress = new Address($params['cart']->id_address_invoice);
            $apiData = $this->getApiDataByAddress($billingAddress);
            if(!empty($apiData)) {
                $defaultCountry = $apiData['country_iso'];
            }
        }

        $this->context->smarty->assign([
            $this->name.'countries_for_banks' => CountryHelper::API_COUNTRIES,
            $this->name.'default_country' => $defaultCountry,
            $this->name.'data_banks' => $dataBanks,
            $this->name.'esto_payment' => PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::BANKLINK],
        ]);

        $htmlBanks = $this->display(__FILE__, 'views/templates/hook/banks.tpl');

        $estoPayments = [
            PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::BANKLINK] => [
                'callToActionText' => !empty($configurations[ConfigKeysHelper::ESTO_PAY_DISPLAY_NAME]) ? $configurations[ConfigKeysHelper::ESTO_PAY_DISPLAY_NAME] : $this->l('ESTO Pay'),
                'additionalInformation' => !empty($configurations[ConfigKeysHelper::ESTO_PAY_DISPLAY_DESCRIPTION]) ? $configurations[ConfigKeysHelper::ESTO_PAY_DISPLAY_DESCRIPTION] : $this->l('Pay instantly with a banklink'),
            ],
            PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::CARD] => [
                'callToActionText' => !empty($configurations[ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_NAME]) ? $configurations[ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_NAME] : $this->l('Pay by card (Visa/Mastercard)'),
                'additionalInformation' => !empty($configurations[ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_DESCRIPTION]) ? $configurations[ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_DISPLAY_DESCRIPTION] : $this->l('Payment is made using a secure payment solution'),
            ],
            PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::REGULAR] => [
                'callToActionText' => !empty($configurations[ConfigKeysHelper::ESTO_DISPLAY_NAME]) ? $configurations[ConfigKeysHelper::ESTO_DISPLAY_NAME] : $this->l('ESTO lease'),
                'additionalInformation' => !empty($configurations[ConfigKeysHelper::ESTO_DISPLAY_DESCRIPTION]) ? $configurations[ConfigKeysHelper::ESTO_DISPLAY_DESCRIPTION] : $this->l('Take a lease from ESTO and let them pay for you. It\'s as quick and simple as paying with a credit card.'),
            ],
            PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::ESTO_X] => [
                'callToActionText' => !empty($configurations[ConfigKeysHelper::ESTO_X_DISPLAY_NAME]) ? $configurations[ConfigKeysHelper::ESTO_X_DISPLAY_NAME] : $this->l('ESTO X'),
                'additionalInformation' => !empty($configurations[ConfigKeysHelper::ESTO_X_DISPLAY_DESCRIPTION]) ? $configurations[ConfigKeysHelper::ESTO_X_DISPLAY_DESCRIPTION] : $this->l('ESTO 3 - pay within three months without additional costs'),
            ],
            PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::PAY_LATER] => [
                'callToActionText' => !empty($configurations[ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_NAME]) ? $configurations[ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_NAME] : $this->l('ESTO Pay later'),
                'additionalInformation' => !empty($configurations[ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_DESCRIPTION]) ? $configurations[ConfigKeysHelper::ESTO_PAY_LATER_DISPLAY_DESCRIPTION] : $this->l('Pay Later'),
            ],
        ];

        $showLogo = Configuration::get(ConfigKeysHelper::ESTO_SHOW_CHECKOUT_LOGO);

        $apiDetails = self::getDefaultCountryApiData();
        $api = new EstoAPI($apiDetails['url'], $apiDetails['user'], $apiDetails['pass'], $apiDetails['country_iso']);
        $availableMethods = $api->getAvailablePaymentMethods();

        foreach ($estoPayments as $estoPayment => $estoPaymentData) {
            if (!$this->validatePaymentOption($estoPayment, $params['cart'], $availableMethods)) {
                continue;
            }

            $logoFileName = Configuration::get($estoPayment . '_CHECKOUT_LOGO', $this->context->language->id);

            if (!$logoFileName) {
                if ($estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::BANKLINK]) {
                    $logoFileName = $this->l('logo-banklink.svg');
                } elseif ($estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::CARD]) {
                    $logoFileName = $this->l('logo-card.svg');
                } elseif ($estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::ESTO_X]) {
                    $logoFileName = $this->l('logo-estox.svg');
                } elseif ($estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::PAY_LATER]) {
                    $logoFileName = $this->l('logo-pay-later.svg');
                } else {
                    $logoFileName = $this->l('logo-regular.svg');
                }
            }

            $smartyVars = [
                'callToActionText' => $estoPaymentData['callToActionText'],
                'additionalInformation' => $estoPaymentData['additionalInformation'],
                'urlToController' => $this->context->link->getModuleLink($this->name, 'payment', ['payment-option' => $estoPayment]),
                'logo' => Media::getMediaPath(_PS_MODULE_DIR_ . $this->name . '/views/img/' . $logoFileName),
                'show_logo' => $showLogo,
                'html_banks' => ($estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::BANKLINK]) ? $htmlBanks : '',
                'estoPayment' => $estoPayment,
            ];

            if ($estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::ESTO_X]) {
                $showEstoXcheckout = Configuration::get(ConfigKeysHelper::ESTO_X_CHECKOUT_CALC);
                $smartyVars['show_esto_x_checkout'] = $showEstoXcheckout;
                if ($showEstoXcheckout) {
                    $image_path =  $this->context->shop->getBaseURL(true) . "/modules/esto/views/img/";
                    $total = $params['cart']->getOrderTotal();
                    $smartyEstoXVars['esto_x_payments'] = $this->getEstoXPayments($total);
                    $smartyEstoXVars['x_calc_logo'] = $smartyVars['logo'];
                    $smartyEstoXVars['image_path'] = $image_path;
                    $this->context->smarty->assign($smartyEstoXVars);
                    $smartyVars['html_esto_x_calc'] = $this->display(__FILE__, 'views/templates/hook/estoxcalc.tpl');
                }
            }

            $this->smarty->assign($smartyVars);

            $htmlPayments .= $this->display(__FILE__, 'payment.tpl');
        }

        return $htmlPayments;
    }

    private function validatePaymentOption($estoPayment, $cart, $availableMethods)
    {
        // Seems unnecessary
        if (!$this->active) {
            return false;
        }

        // Check if this payment method is enabled on ESTO backend
        if (
            $estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::REGULAR] &&
            empty(array_search(PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::REGULAR], $availableMethods, true))
        ) {
            return false;
        }
        if (
            $estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::ESTO_X] &&
            empty(array_search(PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::ESTO_X], $availableMethods, true))
        ) {
            return false;
        }
        if (
            $estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::PAY_LATER] &&
            empty(array_search(PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::PAY_LATER], $availableMethods, true))
        ) {
            return false;
        }
        if (
            $estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::BANKLINK] &&
            empty(array_search(PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::BANKLINK], $availableMethods, true))
        ) {
            return false;
        }
        if (
            $estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::CARD] &&
            empty(array_search(PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::CARD], $availableMethods, true))
        ) {
            return false;
        }
        
        if (
            $estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::REGULAR] &&
            !Configuration::get(ConfigKeysHelper::ESTO_MONTHLY_ENABLED_LANGS_id . $this->context->language->id)
        ) {
            return false;
        }
        if (
            $estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::ESTO_X] &&
            !Configuration::get(ConfigKeysHelper::ESTO_X_IS_ACTIVE)
        ) {
            return false;
        }
        if (
            $estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::PAY_LATER] &&
            !Configuration::get(ConfigKeysHelper::ESTO_PAY_LATER_IS_ACTIVE)
        ) {
            return false;
        }
        if (
            $estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::BANKLINK] &&
            !Configuration::get(ConfigKeysHelper::ESTO_PAY_IS_ACTIVE)
        ) {
            return false;
        }
        if (
            $estoPayment === PaymentMethodHelper::PAYMENT_METHOD[PaymentTypeType::CARD] &&
            !Configuration::get(ConfigKeysHelper::ESTO_PAY_CARD_PAYMENT_IS_ACTIVE)
        ) {
            return false;
        }

        $api_details = self::getDefaultCountryApiData();

        if (
            empty($api_details) ||
            !$api_details['url'] ||
            !$api_details['user'] ||
            !$api_details['pass'] ||
            !$api_details['country_iso'] ||
            !Configuration::get(ConfigKeysHelper::ESTO_CONNECTION_MODE)
        ) {
            return false;
        }

        // Order total must be within the allowed range
        $orderTotal = $cart->getOrderTotal();
        if (PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::REGULAR] === $estoPayment &&
            (
                $orderTotal > Configuration::get(ConfigKeysHelper::ESTO_MAX_ORDER_AMOUNT) ||
                $orderTotal < Configuration::get(ConfigKeysHelper::ESTO_MIN_ORDER_AMOUNT)
            )
        ) {
            return false;
        }
        // The minimum allowed amount for ESTO X is 60€!
        if (
            PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::ESTO_X] === $estoPayment &&
            (
                $orderTotal > Configuration::get(ConfigKeysHelper::ESTO_X_MAX_ORDER_AMOUNT) ||
                $orderTotal < Configuration::get(ConfigKeysHelper::ESTO_X_MIN_ORDER_AMOUNT)
            )
        ) {
            return false;
        }
        if (
            PaymentMethodHelper::PAYMENT_METHOD[PaymentScheduleTypeType::PAY_LATER] === $estoPayment &&
            (
                $orderTotal > Configuration::get(ConfigKeysHelper::ESTO_PAY_LATER_MAX_ORDER_AMOUNT) ||
                $orderTotal < Configuration::get(ConfigKeysHelper::ESTO_PAY_LATER_MIN_ORDER_AMOUNT)
            )
        ) {
            return false;
        }

        return true;
    }

    public function hookDisplayProductPriceBlock($params)
    {
        if (!Configuration::get(ConfigKeysHelper::ESTO_MONTHLY_ENABLED_enable_estomonthly)) {
            return;
        }
        if (!Configuration::get(ConfigKeysHelper::ESTO_MONTHLY_ENABLED_LANGS_id . $this->context->language->id)) {
            return;
        }
        if ($params['type'] != 'price') {
            return;
        }
        if (!isset($params['product']->id)) {
            return;
        }
        $price = Product::getPriceStatic($params['product']->id);
        if (
            max(self::MIN_AMOUNT, floatval(Configuration::get(ConfigKeysHelper::ESTO_MIN_ORDER_AMOUNT))) > $price ||
            min(self::MAX_AMOUNT, floatval(Configuration::get(ConfigKeysHelper::ESTO_MAX_ORDER_AMOUNT))) < $price
        ) {
            return;
        }
        $data = array(
            'amount' => $price
        );

        $api_details = self::getDefaultCountryApiData();
        $api = new EstoAPI($api_details['url'], $api_details['user'], $api_details['pass'], $api_details['country_iso']);
        $response = $api->getMonthlyPayment($data);

        $monthlyLogoFileName = Configuration::get(ConfigKeysHelper::ESTO_MONTHLY_LOGO, $this->context->language->id);
        if (!$monthlyLogoFileName){
            $monthlyLogoFileName = $this->l('logo-regular.svg');
        }

        $this->context->smarty->assign([
            'monthly_payment' => $response,
            'url' => Configuration::get(ConfigKeysHelper::ESTO_MONTHLY_URL, $this->context->language->id),
            'img' => $monthlyLogoFileName,
            'baseurl' => Context::getContext()->shop->getBaseURL(true),
            'desc' => Configuration::get(ConfigKeysHelper::ESTO_MONTHLY_DESCRIPTION, $this->context->language->id),
        ]);

        return $this->display(__FILE__, 'estomonthly.tpl');
    }

    /**
     * @param string $logoName
     * @return string
     */
    public function updateLogo($logoName, $idLang = 0)
    {
        $output = '';

        $logoKeyName = $logoName;
        $logoName = $logoName . '_' . $idLang;
        // Initial check - Is something uploaded at all
        if (!isset($_FILES[$logoName]) || !isset($_FILES[$logoName]['tmp_name']) || empty($_FILES[$logoName]['tmp_name'])) {
            return $output;
        }

        // Upload validation
        $imageValidationError = ImageManager::validateUpload($_FILES[$logoName]);
        if (false !== $imageValidationError) {
            return $output . $this->displayError($this->l('Image validation error') . ' - ' . $logoName);
        }

        // Uploading to a temporary destination
        $oldTmpName = $_FILES[$logoName]['tmp_name'];
        $newTmpName = tempnam(_PS_TMP_IMG_DIR_, 'PS');
        if (!$newTmpName || !move_uploaded_file($oldTmpName, $newTmpName)) {
            return $output . $this->displayError($this->l('Image saving error') . ' - ' . $logoName);
        }

        // Resizing the image uploaded to tmp destination
        // Saving to final destination after resizing
        $destinationFile = dirname(__FILE__) . '/views/img/' . $_FILES[$logoName]['name'];
        $isResizeSuccessful = ImageManager::resize($newTmpName, $destinationFile);
        if (!$isResizeSuccessful) {
            $output .= $this->displayError($this->l('Image resize error') . $logoName);

            // If image is uploaded successfully - update the filename in configuration
        } else {
            Configuration::updateValue($logoKeyName, [$idLang => $_FILES[$logoName]['name']]);
        }

        return $output;
    }

    /**
     * @param string $logoName
     */
    public function deleteLogo($logoName)
    {
        if (intval(Tools::getValue($logoName . '_DELETE_delete')) ||
            intval(Tools::getValue($logoName . '_DELETE'))
        ) {
            $valuesArray = [];
            $activeLangs = Language::getLanguages(true);

            foreach ($activeLangs as $lang) {
                $valuesArray[$lang['id_lang']] = '';
            }
            Configuration::updateValue($logoName, $valuesArray);
        }
    }

    public function updateEnabledLanguages()
    {
        $activeLangs = Language::getLanguages(true);

        foreach ($activeLangs as $lang) {
            Configuration::updateValue(
                ConfigKeysHelper::ESTO_MONTHLY_ENABLED_LANGS_id . $lang['id_lang'],
                Tools::getValue(ConfigKeysHelper::ESTO_MONTHLY_ENABLED_LANGS_id . $lang['id_lang'])
            );
        }
    }

    public static function getCountryApiData($country)
    {
        if (empty(CountryHelper::API_COUNTRIES[$country])) {
            return false;
        }
        $country_iso = CountryHelper::API_COUNTRIES[$country];

        if (empty(EstoAPI::API_ENDPOINTS[$country_iso])) {
            return false;
        }
        $apiEndpoint = EstoAPI::API_ENDPOINTS[$country_iso];

        $esto_shop_id = Configuration::get('ESTO_SHOP_ID_'.$country_iso);
        $esto_secret_key = Configuration::get('ESTO_SECRET_KEY_'.$country_iso);
        if (empty($esto_shop_id) || empty($esto_secret_key)) {
            return false;
        }
        return array(
            'url' => $apiEndpoint,
            'user' => $esto_shop_id,
            'pass' => $esto_secret_key,
            'country_iso' => $country_iso,
        );
    }

    public static function getDefaultCountryApiData()
    {
        $country = Configuration::get(ConfigKeysHelper::ESTO_COUNTRY);

        return self::getCountryApiData($country);
    }

    public static function getApiDataByAddress($address)
    {
        if (empty($address)) {
            return false;
        }

        if (empty(Country::getIsoById($address->id_country))) {
            return false;
        }
        $addressCountryIso = Country::getIsoById($address->id_country);

        return self::getApiDataByCountryCode($addressCountryIso);
    }

    public static function getApiDataByCountryCode($countryIso)
    {
        if (empty($countryIso)) {
            return false;
        }

        $defaultCountry = Configuration::get(ConfigKeysHelper::ESTO_COUNTRY);
        $defaultCountryIso = CountryHelper::API_COUNTRIES[$defaultCountry];

        if ($countryIso == $defaultCountryIso) {
            #pass
        } elseif (!empty(Configuration::get('ESTO_USE_SPECIFIC_ENDPOINT_'.$countryIso))) {
            #pass
        } else {
            return false; # address country is not in the list
        }

        if (empty(CountryHelper::ISO_COUNTRIES[$countryIso])) {
            return false;
        }
        $moduleCountryName = CountryHelper::ISO_COUNTRIES[$countryIso];

        $api_details = self::getCountryApiData($moduleCountryName);

        return $api_details;
    }

    public function update_1_6_10_configuration()
    {
        $selectedCountry = Configuration::get(ConfigKeysHelper::ESTO_COUNTRY);
        if (empty($selectedCountry)) {
            $selectedCountry = RegionType::ESTONIA;
        }

        $selectedCountryIso = CountryHelper::API_COUNTRIES[$selectedCountry];

        $esto_user = Configuration::get(ConfigKeysHelper::ESTO_SHOP_ID);
        if (!empty($esto_user)) {
            if(!$this->checkAndAddConfigurationIfNeeded('ESTO_SHOP_ID_'.$selectedCountryIso, $esto_user)) {
                return false;
            }
        }

        $esto_pass = Configuration::get(ConfigKeysHelper::ESTO_SECRET_KEY);
        if (!empty($esto_pass)) {
            if(!$this->checkAndAddConfigurationIfNeeded('ESTO_SECRET_KEY_'.$selectedCountryIso, $esto_pass)) {
                return false;
            }
        }

        return true;
    }

    public function checkAndAddConfigurationIfNeeded($key, $default = null)
    {
        if (Configuration::hasKey($key)) {
            return true;
        }

        $result = Configuration::updateValue($key, $default);
        return $result;
    }

    protected function validateCredentials($countryName)
    {

        $country_iso = CountryHelper::API_COUNTRIES[$countryName];
        if (empty(strval(Tools::getValue('ESTO_SHOP_ID_'.$country_iso)))){
            return false;
        }
        if (empty(strval(Tools::getValue('ESTO_SECRET_KEY_'.$country_iso)))){
            return false;
        }

        return true;
    }

    protected function validateSpecificCountryCredentials($ee_used, $lv_used, $lt_used)
    {
        $validationResults = [];
        if ($ee_used) {
            $validationResults[] = $this->validateCredentials(RegionType::ESTONIA);
        }

        if ($lv_used) {
            $validationResults[] = $this->validateCredentials(RegionType::LATVIA);
        }

        if ($lt_used) {
            $validationResults[] = $this->validateCredentials(RegionType::LITHUANIA);
        }

        foreach ($validationResults as $valid) {
            if (!$valid) {
                return false;
            }
        }

        return true;
    }

    protected function getEstoXPayments($total)
    {
        $estoXParts = 3;
        $monthNumericFormat = 'n';
        $months = [
            '1' => $this->l('January'),
            '2' => $this->l('February'),
            '3' => $this->l('March'),
            '4' => $this->l('April'),
            '5' => $this->l('May'),
            '6' => $this->l('June'),
            '7' => $this->l('July'),
            '8' => $this->l('August'),
            '9' => $this->l('September'),
            '10' => $this->l('October'),
            '11' => $this->l('November'),
            '12' => $this->l('December'),
        ];


        $total = (int)round($total*100);
        $extraCents = $total % $estoXParts;
        $paymentBase = floor ($total/$estoXParts);

        if (empty($paymentBase)) {
            return [];
        }

        $payments = [];
        $dateTime = new DateTime("now");
        $one_month = new DateInterval('P1M');

        for ($i = 1; $i <= $estoXParts; $i++) {
            $dateTime->add($one_month);
            $amount = $paymentBase;
            if ($i <= $extraCents){
                $amount++;
            }
            $amountFormatted = Tools::displayPrice($amount/100);
            $payments[] = ['amount'=> $amountFormatted, 'month' => $months[$dateTime->format($monthNumericFormat)]];
        }

        return $payments;
    }
}
